"""Console script for summarynb."""
import sys
import click
import os
import stat
import subprocess
import pandas as pd


@click.group()
def main(args=None):
    """Summary Notebooks Tool."""
    pass


def git_root_path():
    # when a hook is executed by git, it's executed in root of repo
    # when we run this as user, we need to manually go to $(git rev-parse --show-toplevel)
    result = subprocess.run(
        ["git", "rev-parse", "--show-toplevel"],
        stdout=subprocess.PIPE,
        universal_newlines=True,
    )
    assert result.returncode == 0, "not a git repo!"
    return result.stdout.strip()


def path_to_config_file():
    return os.path.join(git_root_path(), ".summarynb.config")


def path_to_hook():
    githooks_dir = os.path.join(git_root_path(), ".git/hooks")
    assert os.path.isdir(githooks_dir), "not a git repo!"
    return os.path.join(githooks_dir, "pre-commit")


# TODO: write test that confirms these work identically from git root path as from subdir
# git_root_path()
# path_to_hook()
# path_to_config_file()


# maybe just make this a bash script that runs `summarynb run`?
hook_template = """#!/usr/bin/env python
import sys
from summarynb import cli
if __name__ == '__main__':
    sys.exit(cli.run())
"""


@main.command()
def install():
    fname = path_to_hook()
    # check whether there's already a hook
    if os.path.exists(fname):
        # ask for confirmation
        click.confirm(
            "This will overwrite existing pre-commit hook. Do you want to continue?",
            abort=True,
        )
    with open(fname, "w") as w:
        w.write(hook_template)
        # owner has all permissions including execute
        # group and others have read
        os.chmod(fname, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)
    print("installed")


@main.command()
def uninstall():
    fname = path_to_hook()
    # check whether there's already a hook
    if not os.path.exists(fname):
        print("no hook installed")
        sys.exit(1)
    if click.confirm("This will remove all pre-commit hooks. Do you want to continue?"):
        os.remove(fname)
        print("uninstalled")


def get_or_create_metadata():
    # retrieves or creates metadata file
    fname = path_to_config_file()
    if os.path.exists(fname):
        return pd.read_csv(fname).drop_duplicates()
    return pd.DataFrame(data=None, index=[], columns=["filename"], dtype="object")


def write_metadata(df):
    # writes metadata file
    df.drop_duplicates().to_csv(path_to_config_file(), index=None)


@main.command(name="list")
def list_nb():
    """reports which notebooks are registered for autorun, and whether they are found on disk"""
    df = get_or_create_metadata()
    df["exists"] = df["filename"].apply(os.path.exists)
    print(df)


def prune_nb():
    # TODO: prunes missing notebooks for autorun list
    pass


@main.command()
@click.argument("filepath")
def mark(filepath):
    """registers a notebook for autorun"""
    assert os.path.exists(filepath)
    df = get_or_create_metadata()
    assert not filepath in df["filename"].values, "Already registered"
    df = df.append(pd.DataFrame({"filename": filepath}, index=[-1]))
    write_metadata(df)
    print(
        "Registered notebook for autorun. Make sure git pre-commit hook is installed by running: summarynb install"
    )


@main.command()
@click.argument("filepath")
def unmark(filepath):
    """deregisters a notebook for autorun"""
    df = get_or_create_metadata().set_index("filename")
    # throws exception if filename not in index
    df = df.drop([filepath], axis=0).reset_index()
    write_metadata(df)


@main.command()
def run():
    """
    Execute notebooks in autorun list.

    Don't auto-add to git index - that's an anti-practice for git pre-commit hooks.
    When run locally, give the user a chance to review before committing. And CI should never be adding to the git index.

    Strip away metadata so that this isn't dependent on small changes in python/jupyter versions.
    Equivalent of:
        $ jupyter nbconvert --to notebook --execute Example.ipynb --inplace \
            --ClearMetadataPreprocessor.enabled=True \
                --ClearMetadataPreprocessor.clear_cell_metadata=True \
                    --ClearMetadataPreprocessor.clear_notebook_metadata=True \
                        --ClearMetadataPreprocessor.preserve_nb_metadata_mask='()';
    """
    df = get_or_create_metadata()
    print("Running summary notebooks. Skip this with --no-verify")
    for fname in df["filename"].values:
        print(fname)
        subprocess.run(
            [
                "jupyter",
                "nbconvert",
                "--to",
                "notebook",
                "--execute",
                fname,
                "--inplace",
                "--ClearMetadataPreprocessor.enabled=True",
                "--ClearMetadataPreprocessor.clear_cell_metadata=True",
                "--ClearMetadataPreprocessor.clear_notebook_metadata=True",
                "--ClearMetadataPreprocessor.preserve_nb_metadata_mask=()",
            ]
        )
        print()


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python

import summarynb


def test_image_dimensions():
    assert "max-width: inherit; max-height: inherit;" in summarynb.image("test.png")(
        None, None
    )
    assert "max-width: 800px; max-height: 800px;" in summarynb.image("test.png")(
        800, 800
    )


def test_plaintext():
    assert summarynb.plaintext("Test")() == "<pre>Test</pre>"


def test_textfile():
    assert summarynb.textfile("tests/data/test.txt")() == "<pre>Test from file\n</pre>"


def test_empty():
    assert summarynb.empty(width=None)(800) == '<div style="min-width: 800px;"></div>'
    assert summarynb.empty(width=1600)(800) == '<div style="min-width: 1600px;"></div>'



#!/usr/bin/env python

"""Tests for `summarynb` package."""

import pytest

from click.testing import CliRunner

import summarynb
from summarynb import cli

# TODO: Test _make_HTML() end-to-end against snapshotted reference html
# TODO: Test _get_template()


def test_chunks():
    # should flatten
    # should accept weird shapes
    # really this should be a pytest fixture with separate functions for each, but keeping quick and dirty for now
    test_input = [["a", "b"], ["c", "d"], ["e", "filename"]]
    assert summarynb.chunks(test_input, (3, 2)) == test_input, "Reshape failed"
    assert summarynb.chunks(test_input, (2, 3)) == [
        ["a", "b", "c"],
        ["d", "e", "filename"],
    ], "Reshape failed"
    assert summarynb.chunks(test_input, (1, 6)) == [
        ["a", "b", "c", "d", "e", "filename"]
    ], "Reshape failed"
    assert (
        summarynb.chunks(test_input, (2)) == test_input
    ), "Should accept non-tuple shape"
    assert (
        summarynb.chunks(test_input, 2) == test_input
    ), "Should accept non-tuple shape"
    assert summarynb.chunks(test_input, 4) == [
        ["a", "b", "c", "d"],
        ["e", "filename"],
    ], "Allow mis-shaped row overflow"
    with pytest.raises(AssertionError):
        summarynb.chunks(test_input, (2, 2))


def test_flatten():
    assert (
        summarynb._flatten([["file1", "file2", "file3"]])
        == summarynb._flatten(["file1", "file2", "file3"])
        == summarynb._flatten(["file1", ["file2", ["file3"]]])
        == ["file1", "file2", "file3"]
    )


def test_chunks_accept_flat_list():
    # should accept list, not just list of lists
    assert summarynb.chunks(["file1", "file2", "file3"], 2) == [
        ["file1", "file2"],
        ["file3"],
    ]


def test_list_of_lists_from_object():
    assert summarynb._ensure_list_of_lists("a") == [["a"]]


def test_list_of_lists_from_list():
    assert summarynb._ensure_list_of_lists(["a"]) == [["a"]]


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert "Summary Notebooks Tool" in result.output
    assert "--help  Show this message and exit." in result.output
    help_result = runner.invoke(cli.main, ["--help"])
    assert help_result.exit_code == 0
    assert "--help  Show this message and exit." in help_result.output


def test_read_write_metadata_consistency():
    """Read/write metadata several times. Confirm consistent."""
    df1 = cli.get_or_create_metadata()
    cli.write_metadata(df1)
    df2 = cli.get_or_create_metadata()
    cli.write_metadata(df2)
    df3 = cli.get_or_create_metadata()
    assert df1.equals(df2)
    assert df2.equals(df3)



import os
import pandas as pd
from IPython.display import HTML, display

"""Top-level package for Summary Notebooks."""

__author__ = """Maxim Zaslavsky"""
__email__ = "maxim@maximz.com"
__version__ = "0.1.4"

"""Functions that return functions that return HTML."""

# TODO: make max_width a property of <td>, not <img>?


def image(img_src):
    """Renders an image.

    :param img_src: Image filename.
    :type img_src: str
    :return: Template function that accepts a max pixel width integer and returns HTML.
    :rtype: function
    """

    # Convert absolute path to relative path, since a browser won't be able to grab an image from "/home/..."
    img_src = os.path.relpath(img_src)

    def template(max_width, max_height, *args, **kwargs):
        def convert_to_px_or_unset(optional_numeric_value):
            if not optional_numeric_value:
                return "inherit"
            return str(optional_numeric_value) + "px"

        max_width = convert_to_px_or_unset(max_width)
        max_height = convert_to_px_or_unset(max_height)

        return """<img src="{img_src}" style="max-width: {max_width}; max-height: {max_height};" />""".format(
            img_src=img_src, max_width=max_width, max_height=max_height
        )

    return template


def table(df):
    """Renders a Pandas dataframe.

    :param df: the dataframe
    :type df: pandas.DataFrame
    :return: Template function that returns HTML.
    :rtype: function
    """

    def template(*args, **kwargs):
        return df.to_html()

    return template


def plaintext(text, **kwargs):
    """Renders plain text.

    :param text: text to display
    :type text: str
    """

    def template(*args, **kwargs):
        # convert to html
        return f"<pre>{text}</pre>"

    return template


def empty(width=None):
    """Creates empty separator block.

    :param width: specify block width, defaults to None
    :type width: int, optional
    """

    def template(max_width, *args, **kwargs):
        # empty cell
        return f'<div style="min-width: {max_width if not width else width}px;"></div>'

    return template


"""Extensions that simplify usage of the above base functions."""


def csv(fname, cols=None, **kwargs):
    """
    Read a csv from [fname]. Optionally subset to columns [cols]. Kwargs passed onto pandas read_csv.
    """
    df = pd.read_csv(fname, **kwargs)
    if cols is not None:
        df = df[cols]
    return table(df)


def indexed_csv(fname, cols=None, **kwargs):
    """
    Read a csv from [fname], and set first column as index column. Optionally subset to columns [cols]. Kwargs passed onto pandas read_csv.
    Useful for reading in pandas series quickly.
    """
    return csv(fname, cols=cols, index_col=0, **kwargs)


def textfile(fname, **kwargs):
    """
    Read a text file and render as plain text.
    """
    with open(fname, "r") as f:
        text = f.read()
        return plaintext(text)


"""Main logic."""


def _get_template(user_input):
    """Return executable template function if not provided, based on filename."""
    if callable(user_input):
        # this is already a template function
        return user_input

    # user provided a filename
    # get extension
    extension = os.path.splitext(user_input)[1]

    # detect different table types
    if extension == ".csv":
        return csv(user_input)
    elif extension == ".tsv":
        return csv(user_input, sep="\t")
    elif extension == ".txt":
        return textfile(user_input)

    # assume it's an image
    return image(user_input)


def _ensure_list_of_lists(entries):
    """Transform input to being a list of lists."""
    if not isinstance(entries, list):
        # user passed in single object
        # wrap in a list
        # (next transformation will make this a list of lists)
        entries = [entries]
    if not any(isinstance(element, list) for element in entries):
        # user passed in a list of objects, not a list of lists
        # wrap in a list
        entries = [entries]
    return entries


def _flatten(lst):
    """Flatten sublists in list.
    see https://stackoverflow.com/a/952952/130164
    """
    from collections.abc import Sequence

    entries = []
    for sublist in lst:
        # may be a sublist
        if isinstance(sublist, Sequence) and not isinstance(
            sublist, (str, bytes, bytearray)
        ):
            # recursively flatten sublist
            entries.extend(_flatten(sublist))
        else:
            # just an item
            entries.append(sublist)
    return entries


def chunks(entries, shape):
    """Reshape [entries] into chunks of shape [shape], which can be:
    - a (number of rows, number of columns) tuple, in which case we verify length
    - or simply a number of columns, in which case we guess the right number of rows, and allow in-complete rows."""
    # Flatten entries
    entries = _flatten(entries)

    if isinstance(shape, list) or isinstance(shape, tuple):
        assert len(shape) <= 2, "Only supports 2D arrays"
        if len(shape) == 2:
            # Confirm length
            assert len(entries) == shape[0] * shape[1], "Wrong length."

    # Convert shape to (n_rows, n_cols) tuple.
    if not isinstance(shape, list) and not isinstance(shape, tuple):
        # Input was a single object.
        shape = (0, shape)
    if len(shape) == 1:
        # Input was a list of tuple of length 1.
        shape = (0, shape[0])

    n_col = shape[1]
    reshaped = [
        entries[row_num * n_col : (row_num + 1) * n_col]
        for row_num in range(len(entries) // n_col)
    ]
    if len(entries) % n_col > 0:
        # handle remainder with an uneven row
        reshaped.append(entries[(len(entries) // n_col) * n_col :])
    return reshaped


def _make_HTML(entries, headers, max_width, max_height):
    """
    Create HTML table.
    """

    def wrap_in_column(contents):
        return """<td style="text-align: center">{contents}</td>""".format(
            contents=contents
        )

    def wrap_in_row(contents):
        return """<tr>{contents}</tr>""".format(contents=contents)

    def make_headers(headers):
        if headers is not None:
            return """<tr>{contents}</tr>""".format(
                contents="\n".join(
                    [
                        """<th style="text-align: center">%s</th>"""
                        % str(header).strip()
                        for header in headers
                    ]
                )
            )
        return ""

    def wrap_in_table(contents):
        return """<table>{contents}</table>""".format(contents=contents)

    # Transform to list of lists (list of rows that are each a list of columns), if user passed in single object (1 row, 1 column) or a single list (1 row, many columns)
    entries = _ensure_list_of_lists(entries)

    # Make HTML for each row
    rows = [
        "\n".join(
            [
                wrap_in_column(_get_template(template)(max_width, max_height))
                for template in row
            ]
        )
        for row in entries
    ]
    # assemble rows into table
    return wrap_in_table(
        make_headers(headers) + "\n".join([wrap_in_row(row) for row in rows])
    )


def show(entries, headers=None, max_width=800, max_height=800):
    """
    Display chosen figures and tables in an HTML table.

    Parameters:
        - [entries]: List of rows. Each row is a list of columns. Each column is a list of executable functions that return an HTML template.
        For convenience:
            - Raw filenames (not wrapped in executable functions that return HTML templates) are also accepted, and will be auto-wrapped based on file extension.
            - If a list of lists is not provided, the input is interpreted as forming columns in a single row.

        - [headers]: List of column headers.

        - [max_width]: set max pixel width for images, default 800px. set to None to disable max width.

        - [max_height]: set max pixel height for images, default 800px. set to None to disable max height.
    """

    return display(
        HTML(
            _make_HTML(
                entries, headers=headers, max_width=max_width, max_height=max_height
            )
        )
    )
